#include <iostream>
#include <libtorrent/session.hpp>
#include <libtorrent/alert_types.hpp>

using namespace std;
using namespace libtorrent;

void dispatchAlerts(auto_ptr<alert> alertPtr)
{
    switch (alertPtr->type()) {
    case listen_succeeded_alert::alert_type: {

        listen_succeeded_alert *a = static_cast<listen_succeeded_alert*>(alertPtr.get());
        boost::system::error_code ec;

        string proto = "TCP";
        if (a->sock_type == listen_succeeded_alert::udp)
            proto = "UDP";
        else if (a->sock_type == listen_succeeded_alert::tcp)
            proto = "TCP";
        else if (a->sock_type == listen_succeeded_alert::tcp_ssl)
            proto = "TCP_SSL";
        else
            proto = "OTHER";

        cout << "Listen succeeded on interface: " << a->endpoint.address().to_string(ec).c_str() << " port: " << proto << "/" << a->endpoint.port() << endl;
        break;
    }
    case listen_failed_alert::alert_type: {

        listen_failed_alert *a = static_cast<listen_failed_alert*>(alertPtr.get());
        boost::system::error_code ec;

        string proto = "TCP";
        if (a->sock_type == listen_failed_alert::udp)
            proto = "UDP";
        else if (a->sock_type == listen_failed_alert::tcp)
            proto = "TCP";
        else if (a->sock_type == listen_failed_alert::tcp_ssl)
            proto = "TCP_SSL";
        else
            proto = "OTHER";

        cout << "Listen failed on interface: " << a->endpoint.address().to_string(ec).c_str() << " port: " << proto << "/" << a->endpoint.port() << endl;
        break;
    }

    default:
        break;
    }

}

int main()
{
    fingerprint finger("LT", 1, 0, 8, 0);
    std::pair<int,int> ports(29327, 29327);
    int alertMask = alert::error_notification
                    | alert::peer_notification
                    | alert::port_mapping_notification
                    | alert::storage_notification
                    | alert::tracker_notification
                    | alert::status_notification
                    | alert::ip_block_notification
                    | alert::progress_notification
                    | alert::stats_notification
                    ;

    session nativeSession(finger, ports, 0, 0, alertMask);
    nativeSession.set_alert_dispatch(&dispatchAlerts);

    while (true)
        ;

    return 0;
}
